<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1;

use App\Services\BhOrdersService;
use Illuminate\Http\Request;
use Jiannei\Enum\Laravel\Repositories\Enums\LogEnum;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

class BhOrdersController extends Controller
{

    use Helpers;

    protected $bhOrdersService = null;

    public function __construct(BhOrdersService $bhOrdersService)
    {
       $this->bhOrdersService = $bhOrdersService;
    }

    /**
     * 白化采购订单搜索
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function searchPorders(Request $request)
    {
        $message = [
            'bhOrderNo.required' => "订单号必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $orderList = $this->bhOrdersService->searchPordersToPage($request);
            return Response::success($orderList, '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function editBhPorders(Request $request)
    {
        $message = [
            'poNo.required' => "订单号必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $order = $this->bhOrdersService->editBhPordersInfo($request);
            return Response::success($order, '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 添加一个订单退货人任务
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function addSorderRefundTask(Request $request)
    {
        $message = [
            'name.required' => "任务名称必填",
            'orderNo.required' => "订单号必填",
            'itemId.required' => "订单购买项必填",
            'pId.required' => "订单产品必选",
            'refundNums.required' => "退货数量必填",
            'handleFee.required' => "手续费费用必填",
            'transferPre.required' => "是否转预存必选",
            'cancelPorder.required' => "是否取消采购订单必选",
        ];
        $this->validateRequest($request, $message);

        try{
            $order = $this->bhOrdersService->addSordersRefundTask($request);
            return Response::success($order, '添加成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function editSorderRefundTask(Request $request)
    {
        $message = [
            'taskId.required' => "任务id必填",
            'name.required' => "任务名称必填",
            'orderNo.required' => "订单号必填",
            'itemId.required' => "订单购买项必填",
            'pId.required' => "订单产品必选",
            'refundNums.required' => "退货数量必填",
            'handleFee.required' => "手续费费用必填",
            'transferPre.required' => "是否转预存必选",
            'cancelPorder.required' => "是否取消采购订单必选",
        ];
        $this->validateRequest($request, $message);

        try{
            $order = $this->bhOrdersService->addSordersRefundTask($request);
            return Response::success($order, '编辑成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function delSorderRefundTask(Request $request)
    {
        $message = [
            'taskId.required' => "任务id必传",
        ];
        $this->validateRequest($request, $message);

        try{
            $task = $this->bhOrdersService->delSorderRefundTask($request);
            return Response::success($task, '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 获取订单退货任务列表
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getBhSorderRefundTaskToPage(Request $request)
    {
        $message = [
            'type.required' => "类型无效",
        ];
        $this->validateRequest($request, $message);

        try{
            $task = $this->bhOrdersService->getSordersRefundTask($request);
            return Response::success($task, '添加成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 执行百化退货任务
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function execBhSorderRefundTask(Request $request)
    {
        $message = [
            'taskId.required' => "任务id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $this->bhOrdersService->execSordersRefundTask($request);
            return Response::success([],'操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    /**
     * 查询订单详情
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\JsonResource
     */
    public function getBhSordersDetail(Request $request)
    {
        $message = [
            'orderNo.required' => "订单号必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $order = $this->bhOrdersService->getBhSorderDetail($request);
            return Response::success($this->formatKeysfromArray($order), '更新成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function getSorderRefundTaskLogsToPage(Request $request)
    {
        $message = [
            'taskId.required' => "任务id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $logs = $this->bhOrdersService->getBhSorderTaskLog($request);
            return Response::success($this->formatKeysfromArray($logs), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }

    public function getSorderDetailByTaskId(Request $request)
    {
        $message = [
            'taskId.required' => "任务id必填",
        ];
        $this->validateRequest($request, $message);

        try{
            $logs = $this->bhOrdersService->getBhSorderDetailByTaskId($request);
            return Response::success($this->formatKeysfromArray($logs), '操作成功');
        }catch(\Exception $exception){
            return Response::fail($exception->getMessage(),500);
        }
    }
}
