<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Jobs;

use App\Repositories\Contracts\BhSorderRefundRepository;
use App\Services\BhOrdersService;
use Illuminate\Support\Facades\DB;

class BhSorderRefundJob extends Job
{

    protected $refundInfo;

    public function __construct($refundInfo)
    {
        $this->refundInfo = $refundInfo;
    }

    /**
     * Execute the job.
     */
    public function handle()
    {
        try{

            app(BhSorderRefundRepository::class)->update(['status' => 'running'],$this->refundInfo['id']);

            $note = null;

            $refundInfo = json_decode($this->refundInfo['exec_content']);
            $service = app(BhOrdersService::class);
            $service->sOrderRefund($refundInfo,$note);

            //对任务进行更新
            $update = [];
            $update['status'] = 'finish';
            $update['exec_nums'] = $this->refundInfo['exec_nums'] + 1;
            $update['error_message'] = '';

            if($note && count($note) > 0){
                $update['error_message'] = json_encode($note,JSON_UNESCAPED_UNICODE);
            }

        }catch(\Exception $e){
            //对任务进行更新
            $update = [];
            $update['status'] = 'error';
            $update['error_message'] = $e->getMessage();
            $update['exec_nums'] = intval($this->refundInfo['exec_nums'] + 1);
        }

        try{
            $db = DB::connection('mysql');
            $db->beginTransaction();

            $update['updated_at'] = date('Y-m-d H:i:s',time());
            app(BhSorderRefundRepository::class)->update($update,$this->refundInfo['id']);
            $logInsert = [];
            $logInsert['task_id'] = $this->refundInfo['id'];
            if(isset($update['error_message']) && !empty($update['error_message'])){
                $logInsert['content'] = $update['error_message'];
            }else{
                $logInsert['content'] = '执行完成';
            }
            $logInsert['created_at'] = date('Y-m-d H:i:s',time());
            $logInsert['updated_at'] = date('Y-m-d H:i:s',time());
            $db->table('task_log')->insertGetId($logInsert);

            $db->commit();

        }catch(\Exception $e){
            $db->rollback();
            throw $e;
        }

    }
}
