<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Providers\Auth;

use App\Repositories\Enums\CacheEnum;
use App\Support\Traits\Helpers;
use Illuminate\Auth\AuthenticationException;
use Illuminate\Contracts\Auth\Authenticatable as UserContract;
use Illuminate\Contracts\Auth\UserProvider;
use Illuminate\Contracts\Support\Arrayable;
use Illuminate\Support\Str;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

class AdminUsersProvider implements UserProvider
{
    use Helpers;

    protected $model = null;

    public  function __construct($app, $model)
    {
        if(!$model instanceof \Illuminate\Database\Eloquent\Model){
            $class = '\\'.ltrim($model, '\\');
            $this->model = new $class;
        }else{
            $this->model = $model;
        }
    }

    /**
     * Retrieve a user by their unique identifier.
     *
     * @param  mixed  $identifier
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     */
    public function retrieveById($identifier)
    {
        // 只做根据identifier的值做查询用户操作
        $user = $this->model->newQuery()
            ->where($this->model->getAuthIdentifierName(), $identifier)
            ->first();
        
        if($user->status === 0 ){
            throw new AuthenticationException('用户已被禁用，请联系管理员.');
        }
        
        return $user;

        /*return Cache::remember($cacheKey, $cacheExpireTime, function () use ($identifier) {
            return $this->model->newQuery()
                ->where($this->model->getAuthIdentifierName(), $identifier)
                ->first();
        });*/
    }

    /**
     * Retrieve a user by the given credentials.
     *
     * @param  array  $credentials
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     */
    public function retrieveByCredentials(array $credentials)
    {
        if (empty($credentials)) {
            throw new UnauthorizedHttpException('auth params error','参数缺少',null,ResponseCodeEnum::CLIENT_PARAMETER_ERROR);
        }

        $query = $this->model->newQuery();
        foreach ($credentials as $key => $value) {
            if (Str::contains($key, 'password')) {
                $value = $this->model->encryptPassword($value);
                //continue;
            }
            if (is_array($value) || $value instanceof Arrayable) {
                $query->whereIn($key, $value);
            } else {
                $query->where($key, $value);
            }
        }
        return $query->first();
    }

    /**
     * Validate a user against the given credentials.
     *
     * @param  \Illuminate\Contracts\Auth\Authenticatable  $user
     * @param  array  $credentials
     * @return bool
     */
    public function validateCredentials(UserContract $user, array $credentials)
    {
        $encodePassword = $this->model->encryptPassword($credentials['password']);
        if($encodePassword !== $user->password){
            throw new AuthenticationException('用户登陆密码错误，请重新输入.');
        }
        if($user->status === 0 ){
            throw new AuthenticationException('用户已被禁用，请联系管理员.');
        }
        return true;
    }

    /**
     * Retrieve a user by their unique identifier and "remember me" token.
     *
     * @param  mixed  $identifier
     * @param  string  $token
     * @return \Illuminate\Contracts\Auth\Authenticatable|null
     */
    public function retrieveByToken($identifier, $token)
    {
        return null;
    }

    /**
     * Update the "remember me" token for the given user in storage.
     *
     * @param  \Illuminate\Contracts\Auth\Authenticatable  $user
     * @param  string  $token
     * @return void
     */
    public function updateRememberToken(UserContract $user, $token)
    {
    }
}
