<?php

namespace App\Repositories\Eloquent;

use App\Repositories\Contracts\BhPordersRepository;
use App\Repositories\Criteria\RequestCriteria;
use App\Repositories\Models\BhPorders;
use App\Repositories\Eloquent\BaseRepository;

use Illuminate\Support\Facades\DB;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class BhPordersRepositoryEloquent extends BaseRepository implements BhPordersRepository
{
    protected $fieldSearchable = [
    ];

    /**
     * 定义validator的检索规则
     * @var \string[][]
     */
    public $rules = [
        ValidatorInterface::RULE_CREATE => [
            'menu_name'  => 'required',
            'title'      => 'required',
            'menu_path'  => 'required',
            'parent_id'  => 'required',
            'menu_type'  => 'required',
            'status'  => 'required',
            'sort'  => 'required',
            'is_show'  => 'required',
            'sys_default'  => 'required'
        ],
        ValidatorInterface::RULE_UPDATE => [
            'menu_name'  => 'required',
            'title'      => 'required',
            'menu_path'  => 'required',
            'parent_id'  => 'required',
            'menu_type'  => 'required',
            'status'  => 'required',
            'sort'  => 'required',
            'is_show'  => 'required',
            'sys_default'  => 'required'
        ]
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return BhPorders::class;
    }

    /**
     * Boot up the repository, pushing criteria.
     *
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function boot()
    {
        $this->pushCriteria(app(RequestCriteria::class));
    }


    /**
     * 获取采购订单详情
     * @param $po_id
     * @return array
     */
    public function getPorderItemsFromPoId($po_id,$pi_id = null)
    {
        $pOrder = $this->findWhere(['po_id' => $po_id])->toArray();

        $db = DB::connection('bh_mysql');
        $rowQueryobj = $db->table('poitems')
            ->join('products','poitems.p_id','products.p_id')
            ->join('chemicals','products.c_id','chemicals.c_id')
            ->join('brands','products.b_id','brands.b_id');
        $rowQueryobj->where('poitems.po_id',$po_id);
        if(!is_null($pi_id)){
            $rowQueryobj->where('poitems.pi_id',$pi_id);
        }
        $pOrder['poitems'] = $rowQueryobj->get()->toArray();

        return $pOrder;
    }
}
