<?php

namespace App\Repositories\Eloquent;

use App\Repositories\Contracts\BhSorderRefundRepository;
use App\Repositories\Criteria\RequestCriteria;
use App\Repositories\Models\BhSorderRefund;
use App\Repositories\Eloquent\BaseRepository;

use Illuminate\Support\Facades\DB;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class BhSorderRefundRepositoryEloquent extends BaseRepository implements BhSorderRefundRepository
{
    protected $fieldSearchable = [
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return BhSorderRefund::class;
    }

    /**
     * Boot up the repository, pushing criteria.
     *
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function boot()
    {
        $this->pushCriteria(app(RequestCriteria::class));
    }

    /**
     * 添加退款任务
     * @param $request
     */
    public function addRefundTask($params)
    {
        if(empty($params)){
            throw new \LogicException('参数为空！',500);
        }

        $task = [];
        $task['name'] = $params['name'];
        unset($params['name']);

        $task['exec_content'] = json_encode($params);
        $task['create_by'] = auth()->user()->id;
        $task['status'] = 'notexec';
        $task['type'] = 'sorder_refund';

        if(isset($params['task_id']) && !empty($params['task_id'])){
            $id = $params['task_id'];
            return $this->update($task,$id);
        }else{
            return $this->create($task);
        }
    }

    public function delRefundTask($params)
    {
        if(empty($params)){
            throw new \LogicException('参数为空！',500);
        }

        DB::beginTransaction();
        try{
            $this->delete($params['task_id']);
            DB::table('task_log')
                ->where('task_id',$params['task_id'])
                ->delete();

           DB::commit();
        }catch(\Exception $e){
           DB::rollback();
        }
    }
}
