<?php

namespace App\Repositories\Eloquent;

use App\Repositories\Contracts\RolesRepository;
use App\Repositories\Criteria\RequestCriteria;
use App\Repositories\Eloquent\BaseRepository;

use Prettus\Validator\Contracts\ValidatorInterface;
use Spatie\Permission\Models\Role;

/**
 * Class UserRepositoryEloquent.
 */
class RolesRepositoryEloquent extends BaseRepository implements RolesRepository
{
    protected $fieldSearchable = [
        'name',
    ];

    /**
     * 定义validator的检索规则
     * @var \string[][]
     */
    public $rules = [
        ValidatorInterface::RULE_CREATE => [
            'name'  => 'required',
        ],
        ValidatorInterface::RULE_UPDATE => [
            'name'  => 'required',
        ]
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return Role::class;
    }

    /**
     * Boot up the repository, pushing criteria.
     *
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function boot()
    {
        $this->pushCriteria(app(RequestCriteria::class));
    }

    /**
     * @param $params
     * @return \Illuminate\Database\Eloquent\Builder|\Illuminate\Database\Eloquent\Model
     */
    public function saveRole($params)
    {
        $saveParams = [
            'name' => $params['name'],
            'remark' => $params['remark'],
            'guard_name' => config('auth.defaults.guard'),
        ];
        if(isset($params['id'])){
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());
            //$saveParams['id'] = $params['id'];
            $result = Role::query()->updateOrCreate(['id' => $params['id']],$saveParams);
        }else{
            $saveParams['created_at'] = date('Y-m-d H:i:s',time());
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());
            $result = Role::create($saveParams);
        }

        return $result;
    }
}
