<?php

namespace App\Repositories\Eloquent;

use App\Repositories\Contracts\ToolsTaskRepository;
use App\Repositories\Criteria\RequestCriteria;
use App\Repositories\Eloquent\BaseRepository;

use App\Repositories\Models\SysLog;
use App\Repositories\Models\ToolsTask;
use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class ToolsTaskRepositoryEloquent extends BaseRepository implements ToolsTaskRepository
{
    protected $fieldSearchable = [
    ];

    /**
     * 定义validator的检索规则
     * @var \string[][]
     */
    public $rules = [
        ValidatorInterface::RULE_CREATE => [
            'menu_name'  => 'required',
            'title'      => 'required',
            'menu_path'  => 'required',
            'parent_id'  => 'required',
            'menu_type'  => 'required',
            'status'  => 'required',
            'sort'  => 'required',
            'is_show'  => 'required',
            'sys_default'  => 'required'
        ],
        ValidatorInterface::RULE_UPDATE => [
            'menu_name'  => 'required',
            'title'      => 'required',
            'menu_path'  => 'required',
            'parent_id'  => 'required',
            'menu_type'  => 'required',
            'status'  => 'required',
            'sort'  => 'required',
            'is_show'  => 'required',
            'sys_default'  => 'required'
        ]
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return ToolsTask::class;
    }

    /**
     * Boot up the repository, pushing criteria.
     *
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function boot()
    {
        $this->pushCriteria(app(RequestCriteria::class));
    }

    public function getTaskLogsFromType($type,$task_id=null)
    {
        $taskLogsList = $this
            ->join('task_log','tools_task.id','task_log.task_id')
            ->where('tools_task.type',$type);
        if(!is_null($task_id)){
            $taskLogsList = $taskLogsList->where('tools_task.id',$task_id);
        }
        $taskLogsList = $taskLogsList->get();

        if($taskLogsList){
            return $taskLogsList->toArray();
        }
        return [];
    }
}
