<?php

namespace App\Repositories\Eloquent;

use App\Repositories\Contracts\UsersRepository;
use App\Repositories\Criteria\RequestCriteria;
use App\Repositories\Models\AdminUsers;
use App\Repositories\Validators\UserValidator;
use App\Repositories\Eloquent\BaseRepository;
use App\Support\Traits\Helpers;

use Prettus\Validator\Contracts\ValidatorInterface;

/**
 * Class UserRepositoryEloquent.
 */
class UsersRepositoryEloquent extends BaseRepository implements UsersRepository
{
    use Helpers;
    protected $fieldSearchable = [
        'username','email',
    ];

    /**
     * 定义validator的检索规则
     * @var \string[][]
     */
    public $rules = [
        ValidatorInterface::RULE_CREATE => [
            'username' => 'required|string',
            'nickname' => 'required|string',
            'password'  => 'required|string|min:6',
            'email' => 'required',
            'token' => 'required',
            'is_admin' => 'required',
            'status' => 'required'
        ],
        ValidatorInterface::RULE_UPDATE => [
            'id' => 'required',
            'nickname' => 'required|string',
            'email' => 'required',
        ]
    ];

    /**
     * Specify Model class name.
     *
     * @return string
     */
    public function model()
    {
        return AdminUsers::class;
    }

    /**
     * Boot up the repository, pushing criteria.
     *
     * @throws \Prettus\Repository\Exceptions\RepositoryException
     */
    public function boot()
    {
        $this->pushCriteria(app(RequestCriteria::class));
    }

    public function createOrUpdateUser($params)
    {
        $saveParams = $params;

        if(isset($saveParams['id']) && !empty($saveParams['id'])){
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());
            unset($saveParams['is_admin']);
            unset($saveParams['token']);
            unset($saveParams['username']);
            unset($saveParams['status']);
            return $this->update($saveParams,$saveParams['id']);
        }else{
            $saveParams['is_admin'] = 0;
            $saveParams['status'] = 1;
            $saveParams['token'] = $this->randomFromDevice(16);
            $saveParams['password'] = $this->encryptPassword($saveParams['password']);
            $saveParams['lastlogin'] = 0;
            $saveParams['created_at'] = date('Y-m-d H:i:s',time());
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());
            return $this->create($saveParams);
        }
    }

    public function updateUserStatus($id, $status)
    {
        if($id){
            $saveParams = [];
            $saveParams['status'] = $status;
            $saveParams['id'] = $id;
            $user = $this->where('id',$id)->first()->toArray();

            collect($this->rules['update'])->map(function($rule,$key) use ($user,&$saveParams){
                if(isset($user[$key]) && !empty($user[$key])){
                    $saveParams[$key] = $user[$key];
                }
            });
            $saveParams['status'] = $status;
            $saveParams['id'] = $id;
            $saveParams['updated_at'] = date('Y-m-d H:i:s',time());

            return $this->update($saveParams,$id);
        }
        return true;
    }

    public function deleteUser($ids)
    {
       $users = $this->whereIn('id',$ids)->get();
       if($users){
           collect($users)->map(function($user){
                $user->delete();
           });
       }
       return true;
    }
}
