<?php

use Illuminate\Database\Seeder;
use Illuminate\Support\Facades\DB;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;

use App\Repositories\Models\Menus;
use App\Repositories\Models\AdminUsers;


class PermissionsSeeder extends Seeder
{

    protected $defaultPermissionList = [
        [
            'name' => '查看操作',
            'menu_id' => 0,
            'action' => 'view',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '查看操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '添加操作',
            'menu_id' => 0,
            'action' => 'add',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '添加操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '编辑操作',
            'menu_id' => 0,
            'action' => 'edit',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '编辑操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '删除操作',
            'menu_id' => 0,
            'action' => 'delete',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '删除操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '清空操作',
            'menu_id' => 0,
            'action' => 'clear',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '清空操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '启用操作',
            'menu_id' => 0,
            'action' => 'enable',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '启用操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '禁用操作',
            'menu_id' => 0,
            'action' => 'disable',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '禁用操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '搜索操作',
            'menu_id' => 0,
            'action' => 'search',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '搜索操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '上传操作',
            'menu_id' => 0,
            'action' => 'upload',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '上传操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '导出操作',
            'menu_id' => 0,
            'action' => 'export',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '导出操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '导入操作',
            'menu_id' => 0,
            'action' => 'input',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '导入操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '分配权限操作',
            'menu_id' => 0,
            'action' => 'editPermission',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '分配权限操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '设为默认操作',
            'menu_id' => 0,
            'action' => 'setDefault',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '设为默认操作权限',
            'guard_name' => '',
        ],
        [
            'name' => '其他操作',
            'menu_id' => 0,
            'action' => 'other',
            'permission_type' => 'button',
            'sys_default' => 1,
            'remark' => '其他操作权限',
            'guard_name' => '',
        ],
    ];

    public function run()
    {

        //$this->command->getOutput()->info('正在初始化权限数据中.');

        try{
            DB::beginTransaction();


            collect($this->defaultPermissionList)->map(function($permission){
                $permission['guard_name'] = config('auth.defaults.guard');
                $permission['created_at'] = date('Y-m-d H:i:s',time());
                $permission['updated_at'] = date('Y-m-d H:i:s',time());

                Permission::create($permission);
            });

            // 将初始化菜单进行权限绑定
            $menu = app(Menus::class)->query();
            $menuList = $menu->where('status',1)->get()->toArray();

            collect($menuList)->map(function($menu){
                $permission = [];
                $permission['name'] = $menu['menu_name'];
                $permission['menu_id'] = $menu['id'];
                switch($menu['menu_type']){
                    case 'page' : $permission['action'] = $menu['menu_name'].'view'; break;
                    case 'button' : $permission['action'] = $menu['menu_name']; break;
                    default : $permission['action'] = '';
                }
                $permission['permission_type'] = 'menu';
                $permission['sys_default'] = 0;
                $permission['remark'] = $menu['menu_name'].'的相关权限';
                $permission['guard_name'] = config('auth.defaults.guard');

                Permission::create($permission);
            });

            //给用户组分配权限
            $allPermission = Permission::where('guard_name', config('auth.defaults.guard'))->get();
            $role = Role::all()->first();
            $role->givePermissionTo($allPermission);

            //给用户指定用户组
            $admin = app(AdminUsers::class)->where('is_admin',1)->first();
            $admin->assignRole($role);
            $admin->syncPermissions($allPermission);

            DB::commit();

            $this->command->getOutput()->info('初始化权限数据成功.');

        }catch(Exception $e){
            $this->command->getOutput()->error('初始化权限失败，原因：'.$e->getMessage());
            DB::rollback();
        }
    }
}
